"use strict";Object.defineProperty(exports, "__esModule", {value: true}); function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }



var _chunkYEXUR5PYjs = require('./chunk-YEXUR5PY.js');

// src/index.ts
var _fs = require('fs');
var _unplugin = require('unplugin');

// src/core/options.ts
var _path = require('path');
var _utils = require('@antfu/utils');
var _pluginutils = require('@rollup/pluginutils');
var _haspkg = require('has-pkg'); var _haspkg2 = _interopRequireDefault(_haspkg);
function resolveOptions(options = {}) {
  const imports = flattenImportsMap(options.imports, options.presetOverriding);
  const { dts = _haspkg2.default.call(void 0, "typescript") } = options;
  const resolved = _chunkYEXUR5PYjs.__spreadProps.call(void 0, _chunkYEXUR5PYjs.__spreadValues.call(void 0, {
    sourceMap: false
  }, options), {
    dts: dts === false ? false : dts === true ? _path.resolve.call(void 0, "auto-imports.d.ts") : _path.resolve.call(void 0, dts),
    imports,
    matchRE: new RegExp(`\\b(${Object.keys(imports).join("|")})\\b`, "g"),
    idFilter: _pluginutils.createFilter.call(void 0, options.include || [/\.[jt]sx?$/, /\.vue\??/, /\.svelte$/], options.exclude || [/node_modules/, /\.git/])
  });
  return resolved;
}
function flattenImportsMap(map, overriding = false) {
  const flat = {};
  _utils.toArray.call(void 0, map).forEach((definition) => {
    if (typeof definition === "string") {
      if (!_chunkYEXUR5PYjs.presets[definition])
        throw new Error(`[auto-import] preset ${definition} not found`);
      const preset = _chunkYEXUR5PYjs.presets[definition];
      definition = typeof preset === "function" ? preset() : preset;
    }
    for (const mod of Object.keys(definition)) {
      for (const id of definition[mod]) {
        const meta = {
          module: mod
        };
        if (Array.isArray(id)) {
          meta.name = id[1];
          meta.from = id[0];
        } else {
          meta.name = id;
        }
        if (flat[meta.name] && !overriding)
          throw new Error(`[auto-import] identifier ${meta.name} already defined with ${flat[meta.name].module}`);
        flat[meta.name] = meta;
      }
    }
  });
  return flat;
}

// src/core/transform.ts
var _magicstring = require('magic-string'); var _magicstring2 = _interopRequireDefault(_magicstring);
var excludeRE = [
  /\bimport\s*([\w_$]*?),?\s*\{([\s\S]*?)\}\s*from\b/g,
  /\bfunction\s*([\s\S]+?)\s*\(/g,
  /\b(?:const|let|var)\s*([\w\d_$]+?)\b/g
];
var multilineCommentsRE = /\/\*(.|[\r\n])*?\*\//gm;
var singlelineCommentsRE = /\/\/.*/g;
function stripeComments(code) {
  return code.replace(multilineCommentsRE, "").replace(singlelineCommentsRE, "");
}
function transform(code, id, { matchRE, imports, sourceMap }) {
  const noComments = stripeComments(code);
  const matched = new Set(Array.from(noComments.matchAll(matchRE)).map((i) => i[1]));
  if (!matched.size)
    return null;
  for (const regex of excludeRE) {
    Array.from(noComments.matchAll(regex)).flatMap((i) => {
      var _a, _b;
      return [...((_a = i[1]) == null ? void 0 : _a.split(",")) || [], ...((_b = i[2]) == null ? void 0 : _b.split(",")) || []];
    }).forEach((i) => matched.delete(i.trim()));
  }
  if (!matched.size)
    return null;
  const modules = {};
  Array.from(matched).forEach((name) => {
    var _a;
    const moduleName = (_a = imports[name]) == null ? void 0 : _a.module;
    if (!moduleName)
      return;
    if (!modules[moduleName])
      modules[moduleName] = [];
    modules[moduleName].push(imports[name]);
  });
  const importStatements = Object.entries(modules).map(([moduleName, names]) => {
    const imports2 = names.map(({ name, from }) => from ? `${from} as ${name}` : name).join(", ");
    return `import { ${imports2} } from '${moduleName}';`;
  }).join("");
  const s = new (0, _magicstring2.default)(code);
  s.prependLeft(0, importStatements);
  return {
    code: s.toString(),
    map: sourceMap ? s.generateMap() : null
  };
}

// src/core/dts.ts
function generateDeclration(imports) {
  const body = Object.entries(imports).map(([name, info]) => `  const ${name}: typeof import('${info.module}')['${info.from || name}']`).join("\n");
  return `// Generated by 'unplugin-auto-import'
declare global {
${body}
}
export {}
`;
}

// src/index.ts
var src_default = _unplugin.createUnplugin.call(void 0, (options) => {
  const resolved = resolveOptions(options);
  if (!Object.keys(resolved.imports).length)
    console.warn("[auto-import] plugin installed but no imports has defined, see https://github.com/antfu/unplugin-auto-import#configurations for configurations");
  if (resolved.dts)
    _fs.promises.writeFile(resolved.dts, generateDeclration(resolved.imports), "utf-8");
  return {
    name: "unplugin-auto-import",
    enforce: "post",
    transformInclude(id) {
      return resolved.idFilter(id);
    },
    transform(code, id) {
      return transform(code, id, resolved);
    }
  };
});



exports.src_default = src_default;
