import { Arrayable } from '@antfu/utils';
import { FilterPattern } from '@rollup/pluginutils';

declare const presets: {
    '@nuxtjs/composition-api': ImportsMap;
    '@vue/composition-api': ImportsMap;
    '@vueuse/core': () => ImportsMap;
    '@vueuse/head': ImportsMap;
    preact: ImportsMap;
    react: ImportsMap;
    svelte: ImportsMap;
    'svelte/animate': ImportsMap;
    'svelte/easing': ImportsMap;
    'svelte/motion': ImportsMap;
    'svelte/store': ImportsMap;
    'svelte/transition': ImportsMap;
    'vue-demi': ImportsMap;
    'vue-i18n': ImportsMap;
    'vue-router': ImportsMap;
    vue: ImportsMap;
};
declare type PresetName = keyof typeof presets;

declare type ImportNameAlias = [string, string];
declare type ImportInfo = {
    module: string;
    name: string;
    from?: string;
};
/**
 * module, name, alias
 */
declare type ImportsMap = Record<string, (string | ImportNameAlias)[]>;
/**
 * name, meta
 */
declare type ImportsFlatMap = Record<string, ImportInfo>;
interface Options {
    /**
     * Preset names or custom imports map
     *
     * @default []
     */
    imports?: Arrayable<ImportsMap | PresetName>;
    /**
     * Filepath to generate corresponding .d.ts file.
     * Default enabled when `typescript` is installed locally.
     * Set `false` to disable.
     *
     * @default './auto-imports.d.ts'
     */
    dts?: string | boolean;
    /**
     * Allow overriding imports sources from multiple presets.
     *
     * @default false
     */
    presetOverriding?: boolean;
    /**
     * Rules to include transforming target.
     *
     * @default [/\.[jt]sx?$/, /\.vue\??/]
     */
    include?: FilterPattern;
    /**
     * Rules to exclude transforming target.
     *
     * @default [/node_modules/, /\.git/]
     */
    exclude?: FilterPattern;
    /**
     * Generate source map.
     *
     * @default false
     */
    sourceMap?: boolean;
}
interface TransformOptions {
    imports: ImportsFlatMap;
    matchRE: RegExp;
    /**
     * Generate source map.
     *
     * @default false
     */
    sourceMap?: boolean;
}
interface ResolvedOptions extends Omit<Options, 'imports' | 'dts'>, TransformOptions {
    idFilter: (id: string) => boolean;
    dts: string | false;
}

export { ImportInfo, ImportNameAlias, ImportsFlatMap, ImportsMap, Options, PresetName, ResolvedOptions, TransformOptions };
